function [t,X] = Gillespie(S,kappa,Sub,X0,T,Tlim)
%Gillespie Implementation of Gillespie algorithm
% Inputs:
%   S = stochiometric matrix, n x r
%   kappa = vector of reaction rate constants, 1 x r
%   Sub = reaction substrates matrix, n x r
%   X0 = initial state, n x 1
%   T = time axis limit
%   Tlim = computational time limit
% Outputs:
%   t = vector of time instants, 1 x Nt
%   X = realized process matrix, n x Nt

% computational time limit
% default value 10 s
if nargin == 5
    Tlim = 10;
end

% start clock
tic

% retrieve length
r = length(kappa);

% output variables
t = [];
X = [];

% current state variables
tc = 0;
Xc = X0;

while tc <= T && toc <= Tlim
    
    % add state to output variables
    t = [t tc];
    X = [X Xc];
    
    % compute reaction rates
    lambda = zeros(1,r);
    for k = 1:r
        lambda(k) = ReactionRate(kappa(k),Xc,Sub(:,k));
    end
    
    % transition rate
    lambda0 = sum(lambda);
    
    % transition time
    dt = - log(rand)/lambda0;
    
    % next reaction
    probs = cumsum(lambda)/lambda0;
    K = sum(probs < rand) + 1;
    
    % update state variables
    tc = tc + dt;
    Xc = Xc + S(:,K);
    
end

end
